/*
 * pid.c
 *
 *  Created on: 2022年6月1日
 *      Author: 86137
 */
#include "IQmathLib.h"                    // Include header for IQmath library
#include "pid.h"
#include "Global/Global.h"

//PIDVARS PidVout = {
//
//                      0,                                                  //_iq  Ref;                     // Input: Reference input
//                      0,                                                  //_iq  Fdb;                     // Input: Feedback input
//
//                      0,                                                  //_iq  Err;                     // Variable: Error
//                      0,                                                  //_iq  Err1;                    // History: Previous Error
//                      0,                                                  //_iq  Err2;                    // History: Previous Error
//
//                      0,                                                  //_iq  Up;                      // Variable: Proportional output
//                      0,                                                  //_iq  Ui;                      // Variable: Integral output
//                      0,                                                  //_iq  Ud;                      // Variable: Derivative output
//
//                      PID_Q(VoutMin),                          //_iq  OutMin;         // Parameter: Minimum output
//                      PID_Q(VoutMax),                      //_iq  OutMax;         // Parameter: Maximum output
//
//                      PID_Q(0),                                        //_iq  OutPreSat;      // Variable: Pre-saturated output
//                      PID_Q(0),                                        //_iq  Out;                // Output: PID output
//                      PID_Q(0),                                        //_iq  SatErr;             // Variable: Saturated difference
//
//                      PID_Q(VoltageKp),                       //_iq  Kp;                      // Parameter: Proportional gain
//                      PID_Q(VoltageKi),                       //_iq  Ki;                      // Parameter: Integral gain
//                      PID_Q(VoltageKd),                       //_iq  Kd;                  // Parameter: Derivative gain
//
//                      PID_Q(1),                                       //_iq  Ke;                      // Parameter: Integral threshold value
//                      PID_Q(0),                                       //_iq  Kc;                      // Parameter: Integral correction gain
//
//                      (void (*)(Uint32))PidCalc           //void  (*calc)();          // Pointer to calculation function
//
//               };


//u(k)    =  Kp*(error(k) + Ki*∑error(j) + Kd*(error(k)-error(k-1)))                                             //∑求和:0-K
//u(k-1) =  Kp*(error(k-1) + Ki*∑error(j) + Kd*(error(k-1)-error(k-2)))                                      //∑求和:0-(K-1)
//△u(k)  =  Kp*(error(k)-error(k-1)) + Ki*error(k) + Kd*(error(k)-2*error(k-1)+error(k-2));       //两项相减

void PidCalc(PIDVARS *PidObj)
{

    PidObj->Err[0] = PidObj->Ref - PidObj->Fdb;   // Compute the error          Err=Reference-Feedback
    PidObj->Up = PidObj->Kp*(PidObj->Err[0] - PidObj->Err[1]);    // Compute the proportional output     Up=Kp*(Err[0]-Err[1])
    if(PidObj->Err[0] < PidObj->Ke)                                                                                                     // Compute the integral output             Ui=Ki*Err[0]
    {
        PidObj->Ui = PidObj->Ki*PidObj->Err[0];
    }
    else
    {
        PidObj->Ui = 0;
    }


    PidObj->Ud = PidObj->Kd*(PidObj->Err[0] - 2*PidObj->Err[1] + PidObj->Err[2]);                          // Compute the derivative output           Ud=Kd*(Err[0]-2*Err[1]+Err[2])

    PidObj->OutPreSat = PidObj->Out + PidObj->Up + PidObj->Ui + PidObj->Ud;                              // Compute the pre-saturated output      //OutPreSat=Out+Up+Ui+Ud;

    if (PidObj->OutPreSat >= PidObj->OutMax)                                                                                   // Saturate the output
    {
        PidObj->Out =  PidObj->OutMax;
    }
    else if (PidObj->OutPreSat <= PidObj->OutMin)
    {
        PidObj->Out =  PidObj->OutMin;
    }
    else
    {
        PidObj->Out = PidObj->OutPreSat;
    }

    // Compute the saturate difference0
    //PidObj->SatErr = PidObj->Out - PidObj->OutPreSat;

    // Update the previous proportional output
    PidObj->Err[2] = PidObj->Err[1];
    PidObj->Err[1] = PidObj->Err[0];

}
