/*
 * Temperature.c
 *
 *  Created on: 2022年6月4日
 *      Author: 86137
 */

#include "Temperature.h"



float Rp = 10000;//常温25°下，NTC阻值为10K
float T2 = 298.15;//273.15+25.0;//常温25°的开尔文温度
float BN = 3950.0;//所选NTC的BN值为3950
float T2_BN=1177692.5;
uint8_t time=0;
uint8_t time2=0;
float temp1_total=0;
float temp2_total=0;

/* 函数名：Get_Temp
 * 函数功能：将采集到的ADC电压值，转换成温度值
 * 输入参数说明：adc_channel_t channel 温度传感器ADC通道
 * 返回值：执行成功返回温度值
 * 其他说明：热敏电阻温度转换公式为Rt = R *EXP(B*(1/T1-1/T2))
 * R为常温时的标称值 10K
 * B为热敏电阻的重要参数 3950
 * T1为当前温度(开尔文温度)
 * T2为常温25°温度(开尔文温度) 273.15+25.0
 * Rt为热敏电阻在T1温度下的阻值
 */
float Get_Temp(float val)
{
    float Temp_i;
    float Rt;
    float T;
    float Voltage=val;

    Temp_i=Voltage/2000;//将采集到的ADC电压转换成电流
    Rt=(3.3-Voltage)/Temp_i;//计算NTC热敏电阻阻值 3.3*4095
    T=(BN*T2)/(logf(Rt/Rp)*T2+BN);//阻值与温度转换公式，计算出温度(开尔文温度)
    T=T-273.15;//转换为捏摄氏度

    return T;//
}


void Temp_Get(float *val)
{
    //uint16_t V2[9]={0};
    uint16_t V2[2]={0};
    float Transformation = 0.00080586F;
    R_ADC_Read(&g_adc0_ctrl, channel_temperature1, &V2[0]);//读ADC数据 temp1
    val[7]=V2[0]*Transformation;
    R_ADC_Read(&g_adc0_ctrl, channel_temperature2, &V2[1]);//读ADC数据 temp2
    val[8]=V2[1]*Transformation;
    temp1_total +=Get_Temp(val[7]);
    temp2_total +=Get_Temp(val[8]);
    if(time2>=5)
    {
        time2 =0;
        temperature1 = temp1_total / 5;
        temperature2 = temp2_total / 5;

        temp1_total = 0;
        temp2_total = 0;
    }

    time2 ++;

}


/* 函数名：Temperature_Protect
 * 函数功能：当散热片达到某一温度时，开启散热风扇
 * 输入参数说明：无
 * 返回值：无
 * 由于上散热片的温度比较高，因此温度保护只对上散热片进行处理，即对temperature1处理
 */
void Temperature_Protect(float temp)
{
    if(temp > 35)
    {
        time++;
        if(time >=10)
        {
            R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_02_PIN_07, BSP_IO_LEVEL_LOW);
            time = 0;
        }
    }
    else
        R_IOPORT_PinWrite(&g_ioport_ctrl, BSP_IO_PORT_02_PIN_07, BSP_IO_LEVEL_HIGH);
}
