/*
 * oled.c
 *
 *  Created on: 2021年10月6日
 *      Author: Administrator
 */


#include "oled.h"
#include "oledfont.h"
#include "hal_data.h"
#include "stdint.h"
#include "string.h"

static uint8_t CMD_Data[]={
0xAE,
0x00,
0x10,
0x40,
0xB0,
0x81,
0xFF,
0xA1,
0xA6,
0xA8,
0x3F,
0xC8,
0xD3,
0x00,
0xD5,
0x80,
0xD8,
0x05,
0xD9,
0xF1,
0xDA,
0x12,
0xD8,
0x30,
0x8D,
0x14,
0xAF
};//初始化命令

static uint8_t Mode1_Flag=0,Mode2_Flag=0,Mode3_Flag=0;;

extern fsp_err_t err;
extern int timeout_ms;
extern  i2c_master_event_t i2c_event ;
extern float I_in_bias ;//输入电流采样偏置分量

extern float test;

uint8_t show_time = 0;
float Vin_total = 0;
float Iin_total = 0;
float VDC_total = 0;
float IDC_total = 0;
float Vout_total = 0;
float Iout_total = 0;
float Vin_show = 0;
float Iin_show = 0;
float VDC_show = 0;
float IDC_show = 0;
float Vout_show = 0;
float Iout_show = 0;


//初始化IIC
void Init_IIC(void)
{
    R_IIC_B_MASTER_Open(&g_i2c_master0_ctrl, &g_i2c_master0_cfg);

}

//初始化OLED显示屏
void Init_OLED(void)
{
    uint8_t i = 0;
    uint8_t ii[2]={0x00,0x00};

    for(i=0;i<27;i++)
    {
        ii[1]=CMD_Data[i];
        R_IIC_B_MASTER_Write(&g_i2c_master0_ctrl, ii, 0x02, true);

        /* Since there is nothing else to do, block until Callback triggers*/
        while ((I2C_MASTER_EVENT_TX_COMPLETE != i2c_event) && timeout_ms>0)
        {
            R_BSP_SoftwareDelay(1U, BSP_DELAY_UNITS_MICROSECONDS);
            timeout_ms--;
        }

        if (I2C_MASTER_EVENT_ABORTED == i2c_event)
        {
            IIC_Slave_UnLock();
        }
        /* Read data back from the I2C slave */
        i2c_event = I2C_MASTER_EVENT_ABORTED;
        timeout_ms           = 100;
    }
}

//向设备写控制命令
void OLED_WR_CMD(uint8_t cmd)
{
    uint8_t ii[2]={0x00,0x00};

    ii[1]=cmd;
    R_IIC_B_MASTER_Write(&g_i2c_master0_ctrl, ii, 0x02, true);
    /* Since there is nothing else to do, block until Callback triggers*/
    while ((I2C_MASTER_EVENT_TX_COMPLETE != i2c_event) && timeout_ms>0)
    {
        R_BSP_SoftwareDelay(1U, BSP_DELAY_UNITS_MICROSECONDS);
        timeout_ms--;
    }

    if (I2C_MASTER_EVENT_ABORTED == i2c_event)
    {
        IIC_Slave_UnLock();
    }
    /* Read data back from the I2C slave */
    i2c_event = I2C_MASTER_EVENT_ABORTED;
    timeout_ms           = 100;

}

//向设备写数据
void OLED_WR_DATA(uint8_t data)
{
    uint8_t ii[2]={0x40,0x00};
    ii[1]=data;
    R_IIC_B_MASTER_Write(&g_i2c_master0_ctrl, ii, 0x02, true);
    /* Since there is nothing else to do, block until Callback triggers*/
    while ((I2C_MASTER_EVENT_TX_COMPLETE != i2c_event) && timeout_ms>0)
    {
        R_BSP_SoftwareDelay(1U, BSP_DELAY_UNITS_MICROSECONDS);
        timeout_ms--;
    }
    if (I2C_MASTER_EVENT_ABORTED == i2c_event)
    {
        IIC_Slave_UnLock();
    }
    /* Read data back from the I2C slave */
    i2c_event = I2C_MASTER_EVENT_ABORTED;
    timeout_ms           = 100;
}






//清屏
void OLED_Clear(void)
{
    uint8_t i,n;
    for(i=0;i<8;i++)
    {
        OLED_WR_CMD(0xb0+i);//设置页地址（0~7）
        OLED_WR_CMD (0x00); //设置显示位置—列低地址
        OLED_WR_CMD (0x10); //设置显示位置—列高地址
        for(n=0;n<128;n++)
            OLED_WR_DATA(0);//更新显示
        }



}


//开启OLED显示
void OLED_Display_On(void)
{
    OLED_WR_CMD(0X8D);  //SET DCDC命令
    OLED_WR_CMD(0X14);  //DCDC ON
    OLED_WR_CMD(0XAF);  //DISPLAY ON
}

//关闭OLED显示
void OLED_Display_Off(void)
{
    OLED_WR_CMD(0X8D);  //SET DCDC命令
    OLED_WR_CMD(0X10);  //DCDC OFF
    OLED_WR_CMD(0XAE);  //DISPLAY OFF
}

//坐标设置
void OLED_Set_Pos(uint8_t x, uint8_t y)
{
    OLED_WR_CMD(0xb0+y);
    OLED_WR_CMD(((x&0xf0)>>4)|0x10);
    OLED_WR_CMD(x&0x0f);
}


void OLED_On(void)
{
    uint8_t i,n;
    for(i=0;i<8;i++)
    {
        OLED_WR_CMD(0xb0+i);    //设置页地址（0~7）
        OLED_WR_CMD(0x00);      //设置显示位置—列低地址
        OLED_WR_CMD(0x10);      //设置显示位置—列高地址
        for(n=0;n<128;n++)
            OLED_WR_DATA(1);
    } //更新显示
}

unsigned int oled_pow(uint8_t m,uint8_t n)
{
    unsigned int result=1;
    while(n--)result*=m;
    return result;
}

//显示2个数字
//x,y :起点坐标
//len :数字的位数
//size:字体大小
//mode:模式   0,填充模式;1,叠加模式
//num:数值(0~4294967295);
void OLED_ShowNum(uint8_t x,uint8_t y,unsigned int num,uint8_t len,uint8_t size2)
{
    uint8_t t,temp;
    uint8_t enshow=0;
    for(t=0;t<len;t++)
    {
        temp=(uint8_t)((num/oled_pow(10,len-t-1))%10);
        if(enshow==0&&t<(len-1))
        {
            if(temp==0)
            {
                OLED_ShowChar(x+(size2/2)*t,y,' ',size2);
                continue;
            }else enshow=1;

        }
        OLED_ShowChar(x+(size2/2)*t,y,temp+'0',size2);
    }
}




//在指定位置显示一个字符,包括部分字符
//x:0~127
//y:0~63
//mode:0,反白显示;1,正常显示
//size:选择字体 16/12
void OLED_ShowChar(uint8_t x,uint8_t y,uint8_t chr,uint8_t Char_Size)
{
    unsigned char c=0,i=0;
        c=chr-' ';//得到偏移后的值
        if(x>127)//if(x>128-1)
        {
            x=0;
            y=y+2;
        }
        if(Char_Size ==16)
            {
            OLED_Set_Pos(x,y);
            for(i=0;i<8;i++)
                OLED_WR_DATA(F8X16[c*16+i]);
            OLED_Set_Pos(x,y+1);
            for(i=0;i<8;i++)
                OLED_WR_DATA(F8X16[c*16+i+8]);
            }
            else {
                OLED_Set_Pos(x,y);
                for(i=0;i<6;i++)
                OLED_WR_DATA(F6x8[c][i]);

            }
}


//显示一个字符号串
void OLED_ShowString(uint8_t x,uint8_t y,uint8_t *chr,uint8_t Char_Size)
{
    unsigned char j=0;
    while (chr[j]!='\0')
    {
        OLED_ShowChar(x,y,chr[j],Char_Size);//一次显示一个字符
            x+=8;
        if(x>120){x=0;y+=2;}
        j++;
    }
}








//--------------------------------------------------------------
// Prototype      : void OLED_DrawBMP(unsigned char x0,unsigned char y0,unsigned char x1,unsigned char y1,unsigned char BMP[]);
// Calls          :
// Parameters     : x0,y0 -- 起始点坐标(x0:0~127, y0:0~7); x1,y1 -- 起点对角线(结束点)的坐标(x1:1~128,y1:1~8)
// Description    : 显示BMP位图
//--------------------------------------------------------------
void OLED_DrawBMP(unsigned char x0,unsigned char y0,unsigned char x1,unsigned char y1,unsigned char BMP[])
{
    unsigned int j=0;
    unsigned char x,y;

  if(y1%8==0)
        y = y1/8;
  else
        y = y1/8 + 1;
    for(y=y0;y<y1;y++)
    {
        OLED_Set_Pos(x0,y);
    for(x=x0;x<x1;x++)
        {
        OLED_WR_DATA(BMP[j++]);
        }
    }
}


//double 转 字符串
//void DoubleToStr(double slope, unsigned char str[],int len)
void DoubleToStr(float slope, unsigned char str[],int len)
{
    int t, i;
    int flag = 0; //补足小数点后0的个数
    unsigned char* ptr, buf[5] = { '\0' };  //可以通过增大buf来支持多位数的转换

    ptr = str;
    if (slope < 0) //当数值小于0时，加上负号
    {
        *ptr++ = '-';
        slope = -slope;
    }
    if (slope < 1) //当数值小于1时，加上0
        *ptr++ = '0';
    int temp = (int)slope; //提取整数部分
    i = 0;
 //赋值整数部分
    while (temp > 0)
    {
        t = temp % 10;
        temp = temp / 10;
        buf[i++] =(unsigned char)(t + 0x30); // 通过计算把数字编成ASCII码形式
    }
    i--;
    for (; i >= 0; i--)  // 将得到的字符串倒序
    {
        *(ptr++) = buf[i];
    }
    *(ptr++) = '.';   //小数点
    temp = (int)slope;   //提取整数部分
    slope= slope - (float)temp; //得到小数部分
    slope *= (powf(10,((float)len))); //小数部分放大
    temp = (int)slope;   //提取整数部分
    //补足小数点后的0  //如007，强制转换后会变为7，失去了两个0
    while (len)    //标记失去的0的个数
    {
        len--;
        if (temp < pow(10,len))
            flag++;
    }
    while (flag--)   //在字符串中补足0
        *(ptr++) = '0';
    //赋值小数部分
    i = 0;
    while (temp > 0)
    {
        t = temp % 10;
        temp = temp / 10;
        buf[i++] = (unsigned char)(t + 0x30); // 通过计算把数字编成ASCII码形式
    }
    i--;
    for (; i >= 0; i--)   // 将得到的字符串倒序
        *(ptr++) = buf[i];
}

/************************************************************************************
 * 函数名:OLED_ShowFloat
 * 函数功能:使用OLED显示小数
 * 输入参数:(uint8_t x,uint8_t y,double num,unsigned char len ,uint8_t Char_Size)
 * x:OLED屏幕行坐标位置
 * y:OLED屏幕列坐标位置
 * num：传入小数数值
 * len:小数位数
 * Char_Size:OLED屏幕显示字号大小 可选数值有16 32
 * 函数返回值:无
 * 其他说明:无
 ************************************************************************************/
//void OLED_ShowFloat(uint8_t x,uint8_t y,double num,unsigned char interger,unsigned char decimal ,uint8_t Char_Size)
void OLED_ShowFloat(uint8_t x,uint8_t y,float num,unsigned char interger,unsigned char decimal ,uint8_t Char_Size)
{
    unsigned char temp4[4],temp5[5],temp6[6],temp7[7],temp8[8];//定义多个字符数组，用于存储经函数DoubleToStr转换出来的字符数组，数组长度与小数位数相同，OLED屏幕才不会出现乱码情况
    int count=0;
    count = interger+decimal;//计算数的位数
    switch (count+1)//根据求得的小数位数，+1处理小数点，调用不同的字符数组，防止OLED屏幕乱码。
    {
        case 3:
            DoubleToStr(num,temp4,decimal);
            temp4[3]='\0';
            OLED_ShowString(x,y,temp4,Char_Size);
            break;
        case 4:
            DoubleToStr(num,temp5,decimal);
            temp5[4]='\0';
            OLED_ShowString(x,y,temp5,Char_Size);
            break;
        case 5:
            DoubleToStr(num,temp6,decimal);
            temp6[5]='\0';
            OLED_ShowString(x,y,temp6,Char_Size);
            break;
        case 6:
            DoubleToStr(num,temp7,decimal);
            temp7[6]='\0';
            OLED_ShowString(x,y,temp7,Char_Size);
            break;
        case 7:
            DoubleToStr(num,temp8,decimal);
            temp8[7]='\0';
            OLED_ShowString(x,y,temp8,Char_Size);
            break;
    }
}


void DisPlay()
{
/*  0 8 16 24 32 40 48 56 64 72 80 88 96 104 112 120
 *  M :  0  .  0  0  r  u  n  n  i  n  g
 *  V A  C  :  2  2  0  .  0  0
 *  F r  e  :  4  0  .  0  0  K  D  :  0  .  1   1
 */

    uint16_t V1=0;
    float Transformation = 0.00080586;//0.00080586080586;
    static float a,b;
    Vin_total += V_in;


    R_ADC_Read(&g_adc0_ctrl, channel_Iin, &V1);//读ADC数据 Iin
    ADC_val[5]=((float)V1)*Transformation;
    a= ADC_Iin/(0.8333f);
    b=a-I_in_bias;
    I_in=b/0.04F;
    Iin_total += I_in;

    VDC_total += V_DC;
    IDC_total += I_DC;
    Vout_total += V_out;
    Iout_total += I_out;
    if(show_time >= 10)
    {
        show_time = 0;

        Vin_show = Vin_total /10;
        Iin_show = Iin_total /10;
        VDC_show = VDC_total /10;
        IDC_show = IDC_total /10;
        Vout_show = Vout_total /10;
        Iout_show = Iout_total /10;

        Vin_total = 0;
        Iin_total = 0;
        VDC_total = 0;
        IDC_total = 0;
        Vout_total = 0;
        Iout_total = 0;
    }
    show_time ++;

    switch(Mode)
    {
        case 1:

            if(Mode1_Flag==0)//显示不需要实时更新的字符
            {
                Mode3_Flag=0;
                if(Mode_AC_50Hz == 1)
                    OLED_ShowString(0,0,(uint8_t *)"AC_OUT_50Hz",16);


                OLED_ShowString(0,2,(uint8_t *)"Vi",16);
                OLED_ShowString(72,2,(uint8_t *)"Ii",16);
                

                OLED_ShowString(0,4,(uint8_t *)"Vo",16);
                OLED_ShowString(72,4,(uint8_t *)"Io",16);
                OLED_ShowString(0,6,(uint8_t *)"OC",16);
                OLED_ShowString(32,6,(uint8_t *)"s",16);

                Mode1_Flag=1;

            }else//显示需要实时更新的数据
            {
                //第二行
                if(V_in<10)
                    OLED_ShowFloat(16,2,Vin_show,1,3,16);
                if(V_in>10)
                    OLED_ShowFloat(16,2,Vin_show,2,2,16);


                if(I_in<10)
                    OLED_ShowFloat(88,2,Iin_show*1.11,1,3,16);
                if(I_in>10)
                    OLED_ShowFloat(88,2,Iin_show*1.11,2,2,16);



                //第三行

                if(V_out<10)
                     OLED_ShowFloat(16,4,Vout_show,1,3,16);
                if(V_out>10&&V_out<100)
                     OLED_ShowFloat(16,4,Vout_show,2,2,16);
                if(V_out>100)
                     OLED_ShowFloat(16,4,Vout_show,3,1,16);
                OLED_ShowFloat(88,4,Iout_show,1,3,16);

                //第四行
                if(Over_Current == 1 || Short_Circuit == 1)
                {
                    if(Over_Current == 1)
                    OLED_ShowString(0,6,(uint8_t *)"Over Current",16);
                    if(Short_Circuit == 1)
                        OLED_ShowString(0,6,(uint8_t *)"Short Circuit",16);
                }
                else
                    OLED_ShowNum(16,6,OC_time,2,16);
            }

            break;
        case 2:

            if(Mode2_Flag==0)//显示不需要实时更新的字符
            {
                Mode1_Flag=0;
                //显示模式2代码  预留

                Mode2_Flag=1;
            }
            else//显示需要实时更新的数据
            {

            }

            break;
        case 3:
            if(Mode3_Flag==0)//显示不需要实时更新的字符
            {
                Mode2_Flag=0;
                //显示模式3代码  预留
                Mode3_Flag=1;
            }
            else//显示需要实时更新的数据
            {

            }

            break;
    }

}

/* IIC从机锁死解锁函数 */
void IIC_Slave_UnLock()
{
    uint8_t time = 0;
    R_IOPORT_PinCfg(&g_ioport_ctrl, SDA, IOPORT_CFG_PORT_DIRECTION_INPUT);
    Delay_60Mhz;
    if(Pin_Read(SDA) == BSP_IO_LEVEL_LOW)
    {
        R_IOPORT_PinCfg(&g_ioport_ctrl, SCL, ((uint32_t) IOPORT_CFG_NMOS_ENABLE | (uint32_t) IOPORT_CFG_PORT_DIRECTION_OUTPUT | (uint32_t) IOPORT_CFG_PORT_OUTPUT_LOW));
        for(time = 0;time<9;time++)
        {
            R_IOPORT_PinWrite(&g_ioport_ctrl, SCL, BSP_IO_LEVEL_LOW);
            Delay_60Mhz;
            if(Pin_Read(SDA))
            {
                R_IOPORT_PinWrite(&g_ioport_ctrl, SCL, BSP_IO_LEVEL_HIGH);
                Delay_60Mhz;
            }else
                break;

        }
        if(Pin_Read(SDA) == BSP_IO_LEVEL_HIGH)
        {
            R_IOPORT_PinCfg(&g_ioport_ctrl, SDA, ((uint32_t) IOPORT_CFG_NMOS_ENABLE | (uint32_t) IOPORT_CFG_PORT_DIRECTION_OUTPUT | (uint32_t) IOPORT_CFG_PORT_OUTPUT_LOW));
            R_IOPORT_PinWrite(&g_ioport_ctrl, SDA, BSP_IO_LEVEL_LOW);
            Delay_60Mhz;
            R_IOPORT_PinWrite(&g_ioport_ctrl, SCL, BSP_IO_LEVEL_HIGH);
            Delay_60Mhz;
            R_IOPORT_PinWrite(&g_ioport_ctrl, SDA, BSP_IO_LEVEL_HIGH);
            Delay_60Mhz;
        }
    }
    R_IOPORT_PinCfg(&g_ioport_ctrl, SCL, (IOPORT_CFG_DRIVE_MID | (uint32_t) IOPORT_CFG_PERIPHERAL_PIN | (uint32_t) IOPORT_PERIPHERAL_IIC));
    R_IOPORT_PinCfg(&g_ioport_ctrl, SDA, (IOPORT_CFG_DRIVE_MID | (uint32_t) IOPORT_CFG_PERIPHERAL_PIN | (uint32_t) IOPORT_PERIPHERAL_IIC));
}

bsp_io_level_t Pin_Read(bsp_io_port_pin_t pin)
{
    bsp_io_level_t level=0;
    R_IOPORT_PinRead(&g_ioport_ctrl, pin, &level);
    return level;
}
